/**************************************************************

   Program Title:  IMU64X0 Data Logger
   
   Program Name:  IMU64X0_Controller.c	  Version: 1.00
   
   Author:  SS	  					  Date:  Feb 22, 2013
   
   Compiled with AVR Studio 4.19
   
   This verison is designed for the ATmega1284P processor:
   		128K Flash, 4K EEPROM, 8K RAM
		
   Fuse bit:  FF D9 E2  - Disable JTAG - use internal OSC
   Fuse bit:  FF D9 EE  - Disable JTAG - use external 10MHz Xtal
   Fuse bit:  FF D9 E7  - Disable JTAG - use external 20MHz Xtal

   // If bootloader is installed and starts at 0x1E0000
   Fuse bit:  FF D8 E7  - Disable JTAG - use external 11.0592MHz

   // If bootloader is installed and starts at 0x1F0000
   Fuse bit:  FF DA E7  - Disable JTAG - use external 11.0592MHz

   Lock bit:  EF - With stk500v2.02 bootloader installed - prevents bootloader 
                   from overwriting itself.
   
   Processor support:  ATmega1284P
	
   This program will not run properly if DIV8 fuse bit is set (programmed)
   and JTAG pins are enabled - this is the chip default.
		
   The IMU6410 Monitor is a set of functions that configure IMU6410 AVR internal
   peripherals for interrupt driven operation.  Custom applications can easily
   be built on top to these basic functions.  Included is a complete data 
   acquistion controller with user preferences loaded into EEPROM.  The Monitor
   communicates with the user via an RS-232 port using the CDP.   The AVR is 
   a smart powerful 8 bit RISC processor that can be easily programmed in C
   with on chip TWI, SPI and analog and digital IO.  Functions are included 
   to read/write data blocks to the on board Serial Flash.  On board sensors
   (if included) include a dual axis 2G accelerometer, temperature and light
   sensor and battery voltage monitor.
   
   Key functions:
   	   SPI Master with read/write function to serial flash
	   TWI 2-Wire Master Interrupt Controller
	   Real Time Clock setup with four configurable software timers
	   Full duplex interrupt drive USART with XON/XOFF flow control
	   1msec timer with four configurable software timers
	   Analog controller setup
	   Digital port configuration
	   Accelerometer reader
	   Temperature reader
	   Light level reader
	   IntelHex32 download/upload for EEPROM, AVRFLASH and Serial Flash
	   Auto clock calibration
   
   Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Robotics, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: IMU64X0_Controller.c,v $
 * Revision 1.00  2013/02/22 SS
 *    ZigBee support added
 * Revision 0.99  2012/01/17 SS
 *    IMU6420 support added plus optimization
 * Revision 0.98  2012/01/17 SS
 *    ANT Wireless and Zigbee wireless support
 * Revision 0.97  2012/01/17 SS
 *    Port of DAD and uSD functions plus new logging features
 * Revision 0.95  2011/07/06 SS
 *    Port to AVR Studio 4.19 and IMU6410 focus
 * Revision 0.95  2011/07/06 SS
 *    New support for DOF10W
 * Revision 0.94  2011/11/05 SS
 *    Fixed data logging routines and added support for 10DOFW
 * Revision 0.93  2010/11/05 DL
 *    Added TWI functions to ADS1100 ADC chip
 * Revision 0.92  2010/06/11 RS
 *    Changed to support ImageCraft ICCAVR Standard V7.20.compiler
 * Revision 0.91  2010/03/04 SS
 *    DC200 Support added
 * Revision 0.90  2009/06/20 SS
 *    Serial Flash data logger support for 3-axis rate gyro added
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, Serial Flash writer, XON/XOFF, 9,600baud
 *    real time clock and auto-internal osc calibration.
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 * 
 */ 
// Include header files for GCC/IAR
#include "avr_compiler.h"
#include <avr/pgmspace.h>

#include <string.h>
//#include <stdlib.h>
#include <math.h>
#include <stdio.h>
//#include <AVRdef.h>

#include "IMU64X0_Uart.h"
#include "IMU64X0_Defines.h"
#include "IMU64X0_Type.h"
#include "IMU64X0_Extern.h"
#include "ff.h"
#include "diskio.h"
#include "rtc.h"
#include "xitoa.h"

// ff file system variables
DWORD AccSize;				/* Work register for fs command */
WORD AccFiles, AccDirs;
FILINFO Finfo;
RTC rtc;
#if _USE_LFN
volatile char Lfname[_MAX_LFN+1];
#endif


volatile char Line[100];				/* Console input buffer */

FATFS Fatfs[_VOLUMES];		/* File system object for each logical drive */
FIL File[2];				/* File object */

BYTE Buff[2048];			/* Working buffer */

volatile WORD Timer;		/* 100Hz increment timer */

//  Variable declarations
volatile unsigned int bytesonline;
volatile unsigned char programmode,crc,intelbinary[34];  // length can vary
volatile unsigned char hexbyte[3],hexaddress[5],intelrecordcommand;
volatile unsigned char seconds,minutes,hours,days,month,rtcclockcnt,startloggingmode;
volatile unsigned char flashtype0,flashtype1,flashpageloaded,flashpageboundarytraversed;
volatile unsigned char flashprogrammingactive,flashdatainbuffer,ascii[16];
volatile unsigned char version[30],i2cmode,smartlcdstate,digitalcompassstate,ADSstate;
volatile unsigned char TWI_slaveAddress,TWI_masterAddress,storageselection,microsdname[14],fatmounted;
volatile unsigned char acqtoggle,onemsecond,copyright[38],dadmode,pagemode,usdpresent,printoutput;
volatile unsigned char thresholdmode,thresholdcheck[12],findthresholdaverage,foundthreshold,logrestart;
volatile float thresholdpercent;

volatile unsigned int flashpagesize0,flashpagesize1,pagecnt;
volatile unsigned int intelsuperaddress,oldpage,oldbyteoffset,totalrxcnt;
volatile unsigned int rtccountdown1,rtccountdown2,rtccountdown3,rtccountdown4;
volatile unsigned int mseccountdown1,mseccountdown2,mseccountdown3,mseccountdown4,flashcountdown;
volatile int  acqloopsetting,loopsetting,thresholdloopcnt,logrestartcnt,logrestartmax,year;
volatile int  logrestartcntoffset,usdfilenamenumberoffset;
volatile long dataacqloopcnt;

volatile float max_values[10], min_values[10];
volatile char time_tracker[15];
volatile char max_time_tracker0[15],max_time_tracker1[15],max_time_tracker2[15],max_time_tracker3[15];
volatile char max_time_tracker4[15],max_time_tracker5[15],max_time_tracker6[15],max_time_tracker7[15],max_time_tracker8[15];
volatile char min_time_tracker0[15],min_time_tracker1[15],min_time_tracker2[15],min_time_tracker3[15];
volatile char min_time_tracker4[15],min_time_tracker5[15],min_time_tracker6[15],min_time_tracker7[15],min_time_tracker8[15];
volatile char first_time_grav;  
volatile char first_time_gyro;
volatile char first_time_mag;

volatile long endtimelong,starttimelong;

volatile unsigned char messageBuf[TWIMESSAGELENGTH];


// ************************
// MEMs Sensor Parameters
// ************************

// IMU6410 Parameters
volatile unsigned char logoutputmode,minmax_mode,armpb0,loggingmode,data2log,deviceid,dataformat;
volatile unsigned char deviceidtext[21],deviceidtext2[21],terminationmode,rtccnt;
volatile unsigned char hourstart,minutestart,secondstart,escmode,toggle,usdfilenamemode;
volatile int daystart;
volatile long timeoutperiod,elapsedtime;
volatile char timedstartmode;
volatile int xgmax,xgmin,xgmid,ygmax,ygmin,ygmid,zgmax,zgmin,zgmid;
volatile int defaultlograte,usdfilenamenumber,delaystarttimesec,fsynccnt;

volatile unsigned char srambuf[1024];  // buffer to hold mems sensor data before transferring to flash

// ADXL345 Parameters
volatile unsigned char defaultaccelerometerrange;

// BMA180 Parameters
volatile unsigned char bma180state;

// ITG3200 Parameters
volatile unsigned char itg3200state;

// MS5611 Parameters
volatile unsigned char ms5611state;
volatile unsigned int ms5611_c1,ms5611_c2,ms5611_c3,ms5611_c4,ms5611_c5;
volatile unsigned int ms5611_c6,ms5611_c7,ms5611_c8;

// BMP085 Parameters
volatile unsigned char BMPstate,bmp085mode,bmp085tempstate,bmp085pressurestate,bmp085state;
volatile unsigned int bmpac4,bmpac5,bmpac6,ibmptemperature,ibmppressure;
volatile int  bmpac1,bmpac2,bmpac3,bmpb1,bmpb2,bmpmb,bmpmc,bmpmd;
volatile int  bmp085countdown,bmpskipcount;
volatile long b5;
volatile float bmppressure,bmptemperature,pressuremin,pressuremax;

// L3G4200D Parameters
volatile unsigned char dof10wmode,l3g4200dstate,defaultgyrorange;

// HMC5883 Parameters
volatile unsigned char hmc5883state,defaultmagnetometerrange;


// DAD Variables 
volatile int localStorageFlag,bufferidptr,bytesinpage;
	
volatile int  analogValue0, analogValue1, analogValue2, analogValue3; 
volatile int  analogValue4, analogValue5, analogValue6, analogValue7;

volatile int analogSample;
volatile long flashaddress;

volatile unsigned char MuxPtr;

volatile unsigned char ADC0Resolution,ADC1Resolution,ADC2Resolution;
volatile unsigned char ADC3Resolution,ADC4Resolution,ADC5Resolution;
volatile unsigned char ADC6Resolution,ADC7Resolution;

volatile unsigned char ADC0Exists,ADC1Exists,ADC2Exists,ADC3Exists;
volatile unsigned char ADC4Exists,ADC5Exists,ADC6Exists,ADC7Exists;

volatile unsigned char Digi0Exists,Digi1Exists,Digi2Exists,Digi3Exists;
volatile unsigned char Digi4Exists,Digi5Exists,Digi6Exists,Digi7Exists;
volatile unsigned char Digi8Exists,Digi9Exists,Digi10Exists,Digi11Exists;
volatile unsigned char Digi12Exists,Digi13Exists,Digi14Exists,localStorage;

volatile int StartPin;
volatile int StartPort;
volatile int StartThresholdFlag;
volatile int StartThresholdValue;
volatile int StartThresholdPin;

volatile int StopPin;
volatile int StopPort;
volatile int StopPinFlag;
volatile int StopThresholdFlag;
volatile int StopThresholdValue;
volatile int StopThresholdPin;

volatile unsigned int startCondition;
volatile unsigned int EndCondition;

volatile unsigned int milSecTimer0, milSecTimer1, milSecTimer2, milSecTimer3;
volatile unsigned int milSecTimer4, milSecTimer5, milSecTimer6, milSecTimer7;

volatile int SampleRate0, SampleRate1, SampleRate2, SampleRate3;
volatile int SampleRate4, SampleRate5, SampleRate6, SampleRate7; 

volatile int DigiSampleRate0, DigiSampleRate1, DigiSampleRate2, DigiSampleRate3;
volatile int DigiSampleRate4, DigiSampleRate5, DigiSampleRate6, DigiSampleRate7;
volatile int DigiSampleRate8, DigiSampleRate9, DigiSampleRate10, DigiSampleRate11;
volatile int DigiSampleRate12, DigiSampleRate13, DigiSampleRate14;

volatile unsigned int DigiMilSecTimer0,DigiMilSecTimer1,DigiMilSecTimer2,DigiMilSecTimer3;
volatile unsigned int DigiMilSecTimer4,DigiMilSecTimer5,DigiMilSecTimer6,DigiMilSecTimer7;
volatile unsigned int DigiMilSecTimer8,DigiMilSecTimer9,DigiMilSecTimer10,DigiMilSecTimer11;
volatile unsigned int DigiMilSecTimer12,DigiMilSecTimer13,DigiMilSecTimer14;
// End DAD variables

// Other parameters
volatile unsigned char frameerror,dataoverrun;
volatile unsigned int totalbytecnt,xoncnt,xoffcnt,cursorlocation;
volatile long flashaddress,inteladdress,oldinteladdress;

volatile int _textmode = 0;

//  UART0
volatile unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
volatile unsigned char UART_RxHead;
volatile unsigned char UART_RxTail;
volatile unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
volatile unsigned char UART_TxHead;
volatile unsigned char UART_TxTail;
volatile char  xoffsent, xonsent, enablexonoff, echoinchars,sendxoff;
volatile char  rxcnt,checkflag,rxcntmax;

// UART1
volatile unsigned char UART_RxBuf1[UART_RX_BUFFER_SIZE];
volatile unsigned char UART_RxHead1;
volatile unsigned char UART_RxTail1;
volatile unsigned char UART_TxBuf1[UART_TX_BUFFER_SIZE];
volatile unsigned char UART_TxHead1;
volatile unsigned char UART_TxTail1;
volatile char  xoffsent1, xonsent1, enablexonoff1, echoinchars1,sendxoff1;
volatile char  rxcnt1,checkflag1,rxcntmax1;
volatile unsigned char frameerror1,xoncnt1,xoffcnt1,dataoverrun1,totalrxcnt1;

volatile unsigned char  instrg[8],vlow,vhigh;

volatile unsigned char FlashAddress[3];
const char signon1A[] PROGMEM = "\n\r\n\rIMU6410 ATmega644 Monitor Data Logger V1.00\n\r";
const char signon1B[] PROGMEM = "\n\r\n\rIMU6410 ATmega644P Monitor Data Logger V1.00\n\r";

#ifdef IMU6410
const char signon1C[] PROGMEM = "\n\r\n\rIMU6410 ATmega1284P Monitor Data Logger V1.00\n\r";
#else
const char signon1C[] PROGMEM = "\n\r\n\rIMU6420 ATmega1284P Monitor Data Logger V1.00\n\r";
#endif

const char signon2[] PROGMEM = "(c) Copyright 2004-2013, SOC Robotics, Inc.\n\r";
const char signon3[] PROGMEM = "\n\r   D[E|A|F]  - Display AVR EEPROM, AVR Flash or Flash memory\n\r";
const char signon4[] PROGMEM = "   P[E|A|F]  - Program AVR EEPROM, AVR Flash or Flash memory\n\r";
const char signon5[] PROGMEM = "   L   - Enter log sensor data mode\n\r";
const char signon6[] PROGMEM = "   i   - Enter MEMs sensor output select menu\n\r";
const char signon62[] PROGMEM = "   D   - Return digital compass reading\n\r";
const char signon65[] PROGMEM = "   9[t,r,s]  - Output 9 MEMs data in t-engineering units,r-raw or s-once\n\r";
const char signon66[] PROGMEM = "   10[t,r,s]  - Output 10 MEMs sensor data as above\n\r";
const char signon7[] PROGMEM = "   V   - Version\n\r";
const char signon8[] PROGMEM = "   cb  - Enable/Disable barometer reading on output\n\r";
const char signon83[] PROGMEM = "   T   - Get/Set Time\n\r";
const char signon85[] PROGMEM = "   A   - Read all Analog channels\n\r";
const char signon9[] PROGMEM = "   S   - Signon message\n\r";
const char signon10[] PROGMEM = "   cs  - Change MEMs sensor output rate (msec)\n\r";

const char displaymessage1[] PROGMEM = "Enter start address and length: ";
const char displaymessage2[] PROGMEM = "\n\rDisplay AVR Flash from 0x0054 to 254\n\r";
const char displaymessage3[] PROGMEM = "\n\rDisplay SPI Flash from 0 to 100\n\r";
const char displaymessage33[] PROGMEM = "\n\rNo mode selected\n\r";

const char progmessage[] PROGMEM = "IMU6410 Controller Version: ";

const char displaymessage4[] PROGMEM = "\n\rLoad EEPROM from address ";
const char displaymessage5[] PROGMEM = " length ";
const char displaymessage6[] PROGMEM = "\n\rLoad SPI Flash from address ";

const char loadmessage1[] PROGMEM = "\n\rLoad EEPROM\n\r";
const char loadmessage2[] PROGMEM = "\n\rLoad SPI Flash\n\r";
const char loadmessage3[] PROGMEM = "\n\rLoad AVR Flash\n\r";

// Function declarations
volatile unsigned char EEPROMwrite(int,unsigned char);
volatile unsigned char EEPROMread(int);

volatile unsigned char localecho = NO,convertlftocr=NO;

static volatile int uart_putchar(char c, FILE *stream);

static volatile int uart_getchar(FILE *stream);

static FILE mystdin = FDEV_SETUP_STREAM(NULL, uart_getchar,
                                             _FDEV_SETUP_READ);
static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL,
                                             _FDEV_SETUP_WRITE);

const char mainmess1[] PROGMEM = "\n\rStart logging sensor data\n\r-";
const char mainmess2[] PROGMEM = "\n\rStop logging sensor data\n\r-";
const char mainmess3[] PROGMEM = "Enter cycle rate (msec): ";
const char mainmess4[] PROGMEM = "Enter BMP085 display mode [a|i]: ";
const char mainmess10[] PROGMEM = "w\n\rClear CRA7 bit";
const char mainmess11[] PROGMEM = "`\n\rSet CRA7 bit";
const char mainmess12[] PROGMEM = "\n\r-";

typedef struct {
  double q; //process noise covariance
  double r; //measurement noise covariance
  double x; //value
  double p; //estimation error covariance
  double k; //kalman gain
} kalman_state;

extern kalman_state volatile filtered_pressure;
extern kalman_state kalman_init(double q, double r, double p, double intial_value);
extern delay_time CalculateStartTime(long delaysec);

/**************************************************************************

	Functions

**************************************************************************/
static int uart_getchar(FILE *stream)
{
	unsigned char inchar;

	inchar = GetChar();
	if(localecho==YES) putchar(inchar);

  return (int) inchar;
}

static int uart_putchar(char c, FILE *stream)
{

	  if((c==LF)&&(convertlftocr==YES)) {
	      PutChar(CR);
		  PutChar(c);
		  }
	  else PutChar(c);
	  return 0;
}


#ifdef ATMEGA1284PTARGET
// Older bootloader
void stk500v2_bootloaderB()
{
	 GREEN_LED(OFF);
	 EEPROM_write(1,0x55);  // Load unique byte to hold bootloader

    //  Wait for completion of write
	Delaymsec(30);

 	 asm("jmp 0x1E000");
}

// Latest bootloader 
void stk500v2_bootloader()
{
	 GREEN_LED(OFF);
	 EEPROM_write(1,0x55);  // Load unique byte to hold bootloader

    //  Wait for completion of write
	Delaymsec(30);

 	 asm("jmp 0x1F000");
} 
#endif

#ifdef ATMEGA644TARGET
void stk500v2_bootloader()
{
	 GREEN_LED(OFF);
	 EEPROM_write(1,0x55);  // Load unique byte so hold bootloader

    //  Wait for completion of write
	Delaymsec(30);

 	 asm("jmp 0xf800");
} 
#endif

#ifdef ATMEGA644PTARGET
void stk500v2_bootloader()
{
	 GREEN_LED(OFF);
	 EEPROM_write(1,0x55);  // Load unique byte so hold bootloader

    //  Wait for completion of write
	Delaymsec(30);

 	 asm("jmp 0xf800");
} 
#endif


//*************************************************************
// DAD Command Parser and controller entry point
//*************************************************************
void Check_Main_Loop_led(char ledselect)
{
	// Check toggle condition of Green Led
	if(mseccountdown1==0) {
	    mseccountdown1 = loopsetting;
		if(toggle) {
		    if(ledselect==GREENLED) GREEN_LED(OFF);
			else RED_LED(OFF);
			toggle = 0;
			}
		else {
		    if(ledselect==GREENLED) GREEN_LED(ON);
			else RED_LED(ON);
			toggle = 1;
			}
	    }
}

// Main DAD kernal loop
int DAD_Controller()
{
     unsigned char inchar;
	 
/*	 loopsetting = 300;
	 RED_LED(OFF);
 	 while(dadmode==ACTIVE) {
	     Scheduler();
		// Work starts here
		if((inchar = getcharnw())) 	{   	
		       switch (inchar) 	{	 
					 case ('1'):
						 Dump_Init_Channel_Config();
						 break;
						
					 case ('2'):
						 Write_Test_Data();
						 break;
						 
					 case ('e'):
						 dadmode = INACTIVE;
						 GREEN_LED(OFF);
						 return 1;
						 break;
						 
					 case ('&'):	
						 //printf("Set time [hh:mm:ss]=%d:%d:%d :",hours,minutes,seconds);
					 	 //dataacqloopcnt = 0;
						 if(localStorage==1) {
						     if(FlashBlankCheck()==YES) {
				                 GREEN_LED(ON);
				                 FlashChipErase();
					             GREEN_LED(OFF);
					             }
							 }
						 onemsecond = 0;
						 dataAcquisition();
						 //printf("\n\rdata acq loops = %ld\n\r",dataacqloopcnt);	 
						 //printf("time [hh:mm:ss]=%d:%d:%d :",hours,minutes,seconds);
						 break;
						 
							
					case ('t'):
						 getsettime(QUIET);
						 break;
						 
					case ('T'):
						 getTime();
						 break;
					
					case ('<'):
						ScanInt();
						break; 	
                   							
                    case 'm':
                   		 GREEN_LED(OFF);
                   		 //putcharr('n');
                   		 break;
                   
                    case 'M':
                   		 GREEN_LED(ON);
                   		 //putcharr('N');
                   		 break;
                   	
                   	case 'n':
                   		 RED_LED(OFF);
                   		 //putcharr('m');
                   		 break;
                   	
                    case 'N':
                   		 RED_LED(ON);
                   		 //putcharr('M');
                   		 break;
					
					case 'g':
				    	 GetLoggedData();
						 break;
						 
					case 'K':
				    	 SetStartCondition();
						 break;
					
					case 'k':
				    	 SetEndCondition();
						 break;
						
                	case 'f':
						 channelConfigReceiver();
                	    //testing();
                   		break;
						
					case 'F':
						 Inactivate_Analog();
						 //testing();
                   		break;
						
					case 'i':
						 if(flashtype0==AT45DB081ID) printf("Wasp DAD 1M V0.92\r");
						 else if(flashtype0==AT45DB642ID) printf("Wasp DAD 8M V0.92\r");
						 else printf("Wasp DAD 0F V0.92\r");
                		 break;
						 	
					case 'x':
				         GREEN_LED(ON);
                		 FlashChipErase();
						 GREEN_LED(OFF);
                		 break;	
						 
					case 'H':
                		 localStorage = 1; // local storage on.
						 RED_LED(ON);
                		 break;	
						 
					case 'h':
                		 localStorage = 0; // Local storage off. uses serial instead
						 RED_LED(OFF);
                		 break;	 
					
					case 'Q':
					 	 Inactivate_All_Channels();	// Inactivates sampling of all digital/ADC channels
						 break;
						 
					case 'q':
					 	 Activate_Channel();	// Activates sampling of a channel
						 break;
				 		
		         }// End of switch statement						 
		 }// End of if (for inchar)
	
	  Check_Main_Loop_led(GREENLED);
	  
	}// End of while
*/	return 1;		 	 
}

static
void put_dump (const BYTE *buff, DWORD ofs, BYTE cnt)
{
	BYTE i;

	printf("%08lX ", ofs);

	for(i = 0; i < cnt; i++)
		printf(" %02X", buff[i]);

	putchar(' ');
	for(i = 0; i < cnt; i++)
		putchar((buff[i] >= ' ' && buff[i] <= '~') ? buff[i] : '.');

	printf("\n\r");
}


static
FRESULT scan_files (
	char* path		// Pointer to the working buffer with start path
)
{
	DIR dirs;
	FRESULT res;
	int i;
	char *fn;

	res = f_opendir(&dirs, path);
	if (res == FR_OK) {
		i = strlen(path);
		while (((res = f_readdir(&dirs, &Finfo)) == FR_OK) && Finfo.fname[0]) {
			if (_FS_RPATH && Finfo.fname[0] == '.') continue;
#if _USE_LFN
			fn = *Finfo.lfname ? Finfo.lfname : Finfo.fname;
#else
			fn = Finfo.fname;
#endif
			if (Finfo.fattrib & AM_DIR) {
				AccDirs++;
				*(path+i) = '/'; strcpy(path+i+1, fn);
				res = scan_files(path);
				*(path+i) = '\0';
				if (res != FR_OK) break;
			} else {
				printf("%s/%s\n\r", path, fn);
				AccFiles++;
				AccSize += Finfo.fsize;
			}
		}
	}

	return res;
}

static
void get_line (char *buff, int len)
{
	BYTE c;
	int i = 0;


	for (;;) {
		c = getchar();
		if (c == '\r') break;
		if ((c == '\b') && i) {
			i--;
			putchar(c);
			continue;
		}
		if (c >= ' ' && i < len - 1) {	/* Visible chars */
			buff[i++] = c;
			putchar(c);
		}
	}
	buff[i] = 0;
	putchar('\n');
}

static
void put_rc (FRESULT rc)
{
	const prog_char *p;
	static const prog_char str[] =
		"OK\0" "DISK_ERR\0" "INT_ERR\0" "NOT_READY\0" "NO_FILE\0" "NO_PATH\0"
		"INVALID_NAME\0" "DENIED\0" "EXIST\0" "INVALID_OBJECT\0" "WRITE_PROTECTED\0"
		"INVALID_DRIVE\0" "NOT_ENABLED\0" "NO_FILE_SYSTEM\0" "MKFS_ABORTED\0" "TIMEOUT\0"
		"LOCKED\0" "NOT_ENOUGH_CORE\0" "TOO_MANY_OPEN_FILES\0";
	FRESULT i;

	for (p = str, i = 0; i != rc && pgm_read_byte_near(p); i++) {
		while(pgm_read_byte_near(p++));
	}
	printf("rc=%u FR_%S\n\r", rc, p);
}

char Dump_Directory_Listing()
{
	char inchar=' ',*ptr, *ptr2;
	long p1=0, p2, p3;
	BYTE res, b1, *bp;
	UINT s1, s2, cnt,i;
	DWORD ofs, sect = 0;
	FATFS *fs;
	DIR dir;

	disk_initialize(0);
	f_mount(0, &Fatfs[0]);

	ptr = (char*)Line;
	*ptr = ' ';
	// fl [<path>] - Directory listing
	while (*ptr == ' ') ptr++;
	printf("\n\r");
	res = f_opendir(&dir, ptr);
	if (res) { put_rc(res); return; }
	p1 = s1 = s2 = 0;
	for(;;) {
		res = f_readdir(&dir, &Finfo);
		if ((res != FR_OK) || !Finfo.fname[0]) break;
		if (Finfo.fattrib & AM_DIR) {
			s2++;
		} else {
			s1++; p1 += Finfo.fsize;
		}
		printf("%c%c%c%c%c %u/%02u/%02u %02u:%02u %9lu  %s\n\r", 
					(Finfo.fattrib & AM_DIR) ? 'D' : '-',
					(Finfo.fattrib & AM_RDO) ? 'R' : '-',
					(Finfo.fattrib & AM_HID) ? 'H' : '-',
					(Finfo.fattrib & AM_SYS) ? 'S' : '-',
					(Finfo.fattrib & AM_ARC) ? 'A' : '-',
					(Finfo.fdate >> 9) + 1980, (Finfo.fdate >> 5) & 15, Finfo.fdate & 31,
					(Finfo.ftime >> 11), (Finfo.ftime >> 5) & 63,
					Finfo.fsize, &(Finfo.fname[0]));
#if _USE_LFN
		for (p2 = strlen(Finfo.fname); p2 < 14; p2++)
			putchar(' ');
		printf("%s\n\r", Lfname);
#else
		putchar('\n\r');
#endif
		}
	printf("%4u File(s),%10lu bytes total\n\r%4u Dir(s)", s1, p1, s2);
#if _FS_MINIMIZE < 1
	if (f_getfree(ptr, (DWORD*)&p1, &fs) == FR_OK)
		printf(", %10luK bytes free\n\r", p1 * fs->csize / 2);
#endif
}

int FAT32_FileSystem_Test()
{
	char inchar=' ',*ptr, *ptr2;
	long p1=0, p2, p3;
	BYTE res, b1, *bp;
	UINT s1, s2, cnt,i;
	DWORD ofs, sect = 0;
	FATFS *fs;
	DIR dir;

	printf("\n\rFAT File System Test\n\r");
	if(fatmounted==NO)  printf("Initialize: %d\n\r",disk_initialize((BYTE)p1));
	printf("Enter command: \n\r>");

	while(inchar!='e') {
		ptr = (char*)Line;
		get_line(ptr, sizeof Line);
		inchar = *ptr;
		switch (*ptr++) {

		case 'd' :	// ds <phy_drv#> - Show disk status
			if (!xatoi(&ptr, &p1)) break;
			if (disk_ioctl((BYTE)p1, GET_SECTOR_COUNT, &p2) == RES_OK)
				{ printf("Drive size: %lu sectors\n\r", p2); }
			if (disk_ioctl((BYTE)p1, GET_BLOCK_SIZE, &p2) == RES_OK)
				{ printf("Erase block: %lu sectors\n\r", p2); }
			if (disk_ioctl((BYTE)p1, MMC_GET_TYPE, &b1) == RES_OK)
				{ printf("Card type: %u\n\r", b1); }
			if (disk_ioctl((BYTE)p1, MMC_GET_CSD, Buff) == RES_OK)
				{ printf("CSD: "); put_dump(Buff, 0, 16); }
			if (disk_ioctl((BYTE)p1, MMC_GET_CID, Buff) == RES_OK)
				{ printf("CID: "); put_dump(Buff, 0, 16); }
			if (disk_ioctl((BYTE)p1, MMC_GET_OCR, Buff) == RES_OK)
				{ printf("OCR: "); put_dump(Buff, 0, 4); }
			if (disk_ioctl((BYTE)p1, MMC_GET_SDSTAT, Buff) == RES_OK) {
				printf("SD Status: \n\r");
				for (s1 = 0; s1 < 64; s1 += 16) put_dump(Buff+s1, s1, 16);
			}
			if (disk_ioctl((BYTE)p1, ATA_GET_MODEL, Line) == RES_OK)
				{ Line[40] = '\0'; printf("Model: %s\n\r", Line); }
			if (disk_ioctl((BYTE)p1, ATA_GET_SN, Line) == RES_OK)
				{ Line[20] = '\0'; printf("S/N: %s\n\r", Line); }
			break;

		case 'f' :
			switch (*ptr++) {

			case 'i' :	// fi <log drv#> - Initialize logical drive
				if (!xatoi(&ptr, &p1)) break;
				printf("\r");
				put_rc(f_mount((BYTE)p1, &Fatfs[p1]));
				break;

#if _FS_MINIMIZE < 1
			case 's' :	// fs [<path>] - Show logical drive status
				while (*ptr == ' ') ptr++;
				printf("\r");
				res = f_getfree(ptr, (DWORD*)&p2, &fs);
				if (res) { put_rc(res); break; }
				printf("\n\rFAT type = %u\n\rBytes/Cluster = %lu\n\rNumber of FATs = %u\n\r"
							 "Root DIR entries = %u\n\rSectors/FAT = %lu\n\rNumber of clusters = %lu\n\r"
							 "FAT start (lba) = %lu\n\rDIR start (lba,clustor) = %lu\n\rData start (lba) = %lu\n\r\n\r...",
						fs->fs_type, (DWORD)fs->csize * 512, fs->n_fats,
						fs->n_rootdir, fs->fsize, fs->n_fatent - 2,
						fs->fatbase, fs->dirbase, fs->database
				);
				AccSize = AccFiles = AccDirs = 0;
				res = scan_files(ptr);
				if (res) { put_rc(res); break; }
				printf("\r%u files, %lu bytes.\n\r%u folders.\n\r"
							 "%lu KB total disk space.\n\r%lu KB available.\n\r",
						AccFiles, AccSize, AccDirs,
						(fs->n_fatent - 2) * (fs->csize / 2), p2 * (fs->csize / 2)
				);
				break;

			case 'l' :	// fl [<path>] - Directory listing
				//printf(" - %s -\n\r",ptr);
				while (*ptr == ' ') ptr++;
				printf("\r");
				res = f_opendir(&dir, ptr);

				//printf("&dir=%d\n\r",&dir);

				if (res) { put_rc(res); break; }
				p1 = s1 = s2 = 0;
				for(;;) {
					res = f_readdir(&dir, &Finfo);
					if ((res != FR_OK) || !Finfo.fname[0]) break;
					if (Finfo.fattrib & AM_DIR) {
						s2++;
					} else {
						s1++; p1 += Finfo.fsize;
					}
					printf("%c%c%c%c%c %u/%02u/%02u %02u:%02u %9lu  %s\n\r", 
								(Finfo.fattrib & AM_DIR) ? 'D' : '-',
								(Finfo.fattrib & AM_RDO) ? 'R' : '-',
								(Finfo.fattrib & AM_HID) ? 'H' : '-',
								(Finfo.fattrib & AM_SYS) ? 'S' : '-',
								(Finfo.fattrib & AM_ARC) ? 'A' : '-',
								(Finfo.fdate >> 9) + 1980, (Finfo.fdate >> 5) & 15, Finfo.fdate & 31,
								(Finfo.ftime >> 11), (Finfo.ftime >> 5) & 63,
								Finfo.fsize, &(Finfo.fname[0]));
#if _USE_LFN
					for (p2 = strlen(Finfo.fname); p2 < 14; p2++)
						putchar(' ');
					printf("%s\n\r", Lfname);
#else
					putchar('\n\r');
#endif
				}
				printf("%4u File(s),%10lu bytes total\n\r%4u Dir(s)", s1, p1, s2);
				if (f_getfree(ptr, (DWORD*)&p1, &fs) == FR_OK)
					printf(", %10luK bytes free\n\r", p1 * fs->csize / 2);
				break;
#endif

			case 'o' :	// fo <mode> <name> - Open a file  ( set mode=1 to read)
				if (!xatoi(&ptr, &p1)) break;
				printf("\r");
				while (*ptr == ' ') ptr++;
				put_rc(f_open(&File[0], ptr, (BYTE)p1));
				break;

			case 'c' :	// fc - Close a file
				printf("\r");
				put_rc(f_close(&File[0]));
				break;

			case 'e' :	// fe - Seek file pointer
				if (!xatoi(&ptr, &p1)) break;
				res = f_lseek(&File[0], p1);
				put_rc(res);
				if (res == FR_OK)
					printf("fptr = %lu(0x%lX)\n\r", File[0].fptr, File[0].fptr);
				break;

			case 'd' :	// fd <len> - read and dump file from current fp
				if (!xatoi(&ptr, &p1)) break;
				printf("\r");
				ofs = File[0].fptr;
				while (p1) {
					if (p1 >= 16)	{ cnt = 16; p1 -= 16; }
					else 			{ cnt = (WORD)p1; p1 = 0; }
					res = f_read(&File[0], Buff, cnt, &cnt);
					if (res != FR_OK) { put_rc(res); break; }
					if (!cnt) break;
					for(i=0; i<cnt; i++) putchar(Buff[i]);
					//put_dump(Buff, ofs, cnt);
					ofs += 16;
				}
				break;

/*			case 'w' :	// fw <len> <val> - write file
				if (!xatoi(&ptr, &p1) || !xatoi(&ptr, &p2)) break;
				printf("\r");
				memset(Buff, (BYTE)p2, sizeof Buff);
				p2 = 0;
				mseccountdown4 = 10000;
				while (p1) {
					if (p1 >= sizeof Buff)	{ cnt = sizeof Buff; p1 -= sizeof Buff; }
					else 			{ cnt = (WORD)p1; p1 = 0; }
					res = f_write(&File[0], Buff, cnt, &s2);
					if (res != FR_OK) { put_rc(res); break; }
					p2 += s2;
					if (cnt != s2) break;
				}
				//s2 = Timer;
				printf("%lu bytes written in %d msec.\n\r", p2, 10000-mseccountdown4);
				break;

			case 'v' :	// fv - Truncate file
				put_rc(f_truncate(&File[0]));
				break;
*/
#if _FS_MINIMIZE < 1
			case 'n' :	// fn <old_name> <new_name> - Change file/dir name
				while (*ptr == ' ') ptr++;
				printf("\r");
				ptr2 = strchr(ptr, ' ');
				if (!ptr2) break;
				*ptr2++ = 0;
				while (*ptr2 == ' ') ptr2++;
				put_rc(f_rename(ptr, ptr2));
				break;

			case 'u' :	// fu <name> - Unlink a file or dir
				while (*ptr == ' ') ptr++;
				printf("\r");
				put_rc(f_unlink(ptr));
				break;
#endif

/*			case 'k' :	// fk <name> - Create a directory
				while (*ptr == ' ') ptr++;
				put_rc(f_mkdir(ptr));
				break;
*/
			case 'x' : // fx <src_name> <dst_name> - Copy file
				while (*ptr == ' ') ptr++;
				printf("\r");
				ptr2 = strchr(ptr, ' ');
				if (!ptr2) break;
				*ptr2++ = 0;
				while (*ptr2 == ' ') ptr2++;
				printf("Opening \"%s\"", ptr);
				res = f_open(&File[0], ptr, FA_OPEN_EXISTING | FA_READ);
				if (res) {
					put_rc(res);
					break;
				}
				printf("\n\rCreating \"%s\"", ptr2);
				res = f_open(&File[1], ptr2, FA_CREATE_ALWAYS | FA_WRITE);
				if (res) {
					put_rc(res);
					f_close(&File[0]);
					break;
				}
				printf("\n\rCopying...");
				p1 = 0;
				for (;;) {
					res = f_read(&File[0], Buff, sizeof Buff, &s1);
					if (res || s1 == 0) break;   // error or eof
					res = f_write(&File[1], Buff, s1, &s2);
					p1 += s2;
					if (res || s2 < s1) break;   // error or disk full
				}
				if (res) put_rc(res);
				printf("\n\r%lu bytes copied.\n\r", p1);
				f_close(&File[0]);
				f_close(&File[1]);
				break;
/*
#if _FS_RPATH
			case 'g' :	// fg <path> - Change current directory
				while (*ptr == ' ') ptr++;
				put_rc(f_chdir(ptr));
				break;

			case 'j' :	// fj <drive#> - Change current drive
				if (xatoi(&ptr, &p1)) {
					put_rc(f_chdrive((BYTE)p1));
				}
				break;
#if _FS_RPATH >= 2
			case 'q' :	// fq - Show current dir path
				res = f_getcwd(Line, sizeof Line);
				if (res)
					put_rc(res);
				else
					printf("%s\n\r", Line);
				break;
#endif
#endif
*/
#if _USE_MKFS
			case 'm' :	// fm <logi drv#> <part type> <bytes/clust> - Create file system
				if (!xatoi(&ptr, &p1) || !xatoi(&ptr, &p2) || !xatoi(&ptr, &p3)) break;
				printf("\r");
				printf("The drive %u will be formatted. Are you sure? (Y/n)=", (WORD)p1);
				get_line(ptr, sizeof Line);
				if (*ptr == 'Y') put_rc(f_mkfs((BYTE)p1, (BYTE)p2, (WORD)p3));
				break;
#endif
			}
			break;
			}
		if(inchar!='e')  printf("\n\r>");
		}
	return 0;
}

void MountCreateFATFile()
{
	if(fatmounted==NO) {
	    f_mount(0, &Fatfs[0]);
		fatmounted = YES;
		}
    if(usdfilenamemode==INCREMENTAL) {
	    RED_LED(ON);
	    usdfilenamenumber = ReadEepromInt(usdfilenamenumberoffset);

		//printf("\n\rusdfilenamenumber=%d\n\r",usdfilenamenumber);
		//printf("usdfilenamenumberoffset = %d\n\r",usdfilenamenumberoffset);

	    sprintf(microsdname,"imu%05d.txt",usdfilenamenumber++);

		//printf("usd filename=%s\n\r",microsdname);

		WriteEepromInt(usdfilenamenumberoffset,usdfilenamenumber); 

		//printf("usdfilenamenumber=%d\n\r",usdfilenamenumber);

		}
	if(f_open(&File[0], microsdname, FA_WRITE)==FR_NO_FILE) {
	    //printf(" file %s doesn't exist\n\r",microsdname);
	    f_open(&File[0], microsdname, FA_CREATE_NEW);
		f_open(&File[0], microsdname, FA_WRITE);
		}
	RED_LED(OFF);
}

volatile char togglemode;

void Check_Green_Led()
{
	if(mseccountdown1==0) {
		mseccountdown1 = 50;
		if(togglemode)  {
		    GREEN_LED(ON);
			togglemode = 0;
			}
		else {
			GREEN_LED(OFF);
			togglemode = 1;
			}
		}
}

int Log_2_MicroSD(char logmode)
{
	char sendString[60];
	char inchar,inchar2;
	delay_time starttime;
	UINT bw;         /* File read/write count */

	if((usdpresent==YES)&&(storageselection==MICROSD))  MountCreateFATFile();
	if((logrestart==NO)||((logrestart==YES)&&(logrestartcnt==0))) {
		if(timedstartmode==YES) {
		    while(CheckStartTime()==NO) {
			    printf("Logging starts at %2d:%2d:%2d now %2d:%2d:%2d   \r",
				         hourstart,minutestart,secondstart,hours,minutes,seconds);
				if(getcharnw() && getcharnw1()) return 0;
				}
			}
		if(thresholdmode==ACTIVE) {
		    findthresholdaverage = YES;
			foundthreshold = NO;
			thresholdloopcnt = 0;

			printf("Threshold mode active - waiting for threshold\n\r");
			}
		}
	if(terminationmode==TIMEOUT) rtccountdown4 = timeoutperiod;

	Init_BMP085_Readings();
	
	first_time_grav = YES;  
	first_time_gyro = YES;
	first_time_mag = YES;

RESTART:
	while((inchar=getcharnw())!='e' && (inchar2=getcharnw1())!='e') {
	    mseccountdown3 = defaultlograte;

		//printoutput = NO;
		//printf("rtccountdown4=%d   \r",rtccountdown4);

		DisplayDOF10WData(logmode);
		first_time_grav = NO;  
		first_time_gyro = NO;
		first_time_mag = NO;
		while(mseccountdown3) ;
		Check_Green_Led();
		if(inchar2=='s') break;

		// Quite after termination timeout if active
		if(terminationmode==TIMEOUT) {
		    if(!rtccountdown4) break;
			}
		}
	loggingmode = HALT;
	if(minmax_mode == 'e')  {
		//max_values
		Buff[0] = 0;  // reset buffer
		strcat(Buff,"\r\n\r\nMax values:\r\nAttribute      Value         Time\r\n");

		//sprintf(sendString,"%#6.3f %#6.3f %#6.3f ",max_values[0], max_values[1], max_values[2]);
		sprintf(sendString,"X.Accel       %#6.3f     %s\r\n",max_values[0], max_time_tracker0);
		strcat(Buff,sendString);
		sprintf(sendString,"Y.Accel       %#6.3f     %s\r\n",max_values[1], max_time_tracker1);
		strcat(Buff,sendString);
		sprintf(sendString,"Z.Accel       %#6.3f     %s\r\n",max_values[2], max_time_tracker2);
		strcat(Buff,sendString);

		//sprintf(sendString,"%#6.1f %#6.1f %#6.1f ",max_values[3], max_values[4], max_values[5]);
		sprintf(sendString,"X.Gyro        %#6.1f     %s\r\n",max_values[3], max_time_tracker3);
		strcat(Buff,sendString);
		sprintf(sendString,"Y.Gyro        %#6.1f     %s\r\n",max_values[4], max_time_tracker4);
		strcat(Buff,sendString);
		sprintf(sendString,"Z.Gyro        %#6.1f     %s\r\n",max_values[5], max_time_tracker5);
		strcat(Buff,sendString);

		//sprintf(sendString,"%#6.3f %#6.3f %#6.3f ",max_values[6], max_values[7], max_values[8]);
		sprintf(sendString,"X.Mag         %#6.3f     %s\r\n",max_values[6], max_time_tracker6);
		strcat(Buff,sendString);
		sprintf(sendString,"Y.Mag         %#6.3f     %s\r\n",max_values[7], max_time_tracker7);
		strcat(Buff,sendString);
		sprintf(sendString,"Z.Mag         %#6.3f     %s\r\n",max_values[8], max_time_tracker8);
		strcat(Buff,sendString);

		//min_values
		strcat(Buff,"\rMin values:\rAttribute      Value         Time\r");

		//sprintf(sendString,"%#6.3f %#6.3f %#6.3f ",min_values[0], min_values[1], min_values[2]);
		sprintf(sendString,"X.Accel       %#6.3f     %s\r\n",min_values[0], min_time_tracker0);
		strcat(Buff,sendString);
		sprintf(sendString,"Y.Accel       %#6.3f     %s\r\n",min_values[1], min_time_tracker1);
		strcat(Buff,sendString);
		sprintf(sendString,"Z.Accel       %#6.3f     %s\r\n",min_values[2], min_time_tracker2);
		strcat(Buff,sendString);

		//sprintf(sendString,"%#6.1f %#6.1f %#6.1f ",min_values[3], min_values[4], min_values[5]);
		sprintf(sendString,"X.Gyro        %#6.1f     %s\r\n",min_values[3], min_time_tracker3);
		strcat(Buff,sendString);
		sprintf(sendString,"Y.Gyro        %#6.1f     %s\r\n",min_values[4], min_time_tracker4);
		strcat(Buff,sendString);
		sprintf(sendString,"Z.Gyro        %#6.1f     %s\r\n",min_values[5], min_time_tracker5);
		strcat(Buff,sendString);

		//sprintf(sendString,"%#6.3f %#6.3f %#6.3f ",min_values[6], min_values[7], min_values[8]);
		sprintf(sendString,"X.Mag         %#6.3f     %s\r\n",min_values[6], min_time_tracker6);
		strcat(Buff,sendString);
		sprintf(sendString,"Y.Mag         %#6.3f     %s\r\n",min_values[7], min_time_tracker7);
		strcat(Buff,sendString);
		sprintf(sendString,"Z.Mag         %#6.3f     %s\r\n",min_values[8], min_time_tracker8);
		strcat(Buff,sendString);
		}

	f_write(&File[0], Buff, strlen(Buff), &bw);

	if(storageselection==MICROSD) {
	    f_close(&File[0]);
		printf("Close file %s",microsdname);
		}


	// Disable log restart if user exits logging
	if((inchar=='e')||(inchar2=='e')||(logrestart==NO)) {
	    logrestart = NO;
		startloggingmode = NOTONPOWERUP;
		loggingmode = HALT;
		}
	else inchar = ' ';

	// If logrestart mode active increment file count and start again
	if(logrestart==YES) {
		logrestartcnt++;
		if(logrestartcnt<logrestartmax) {

		    printf("\n\rRestart cycle %d of %d\n\r",logrestartcnt+1,logrestartmax);

			if(terminationmode==TIMEOUT) rtccountdown4 = timeoutperiod;
			WriteEepromInt(logrestartcntoffset,logrestartcnt);
	        MountCreateFATFile();
		    goto RESTART;
			}
		}
	logrestartcnt = 0;
	WriteEepromInt(logrestartcntoffset,logrestartcnt);
	startloggingmode = NOTONPOWERUP;
	return 0;
}

// Check if logging needs to start, needs to log or needs to stop
// This routine handles both Serial Flash and microSD
void CheckLogging(char inchar)
{
	int delaycount;

	// Handle serial flash start/stop conditions 
	//   keyboard start, PB0 start, timed start and timed stop
	if(storageselection==SERIALFLASH) {
		// The log data function is started
		if(loggingmode==START) {
		    LogData(defaultlograte);
			if((terminationmode==TIMEOUT)&&(rtccountdown4==0)) {
			    GREEN_LED(ON);
				Delaymsec(3000);
				loggingmode = HALT;
			    //printf("\n\rStop logging sensor data\n\r-");
				putfs(mainmess2);
			    GREEN_LED(OFF);
				}
			}
		
		// Turn logging off if button pressed again
		if((armpb0==YES)&&(!(PINB&(1<<PINB0)))&&(loggingmode==START)) {
		    delaycount = 0;
			GREEN_LED(ON);
		    while((!(PINB&(1<<PINB0)))&&(delaycount<4000)) {
				delaycount += 10;
			    Delaymsec(10);
				}
				
			//printf("turn off ");
			// If push button high haven't waited long enough
			if(delaycount>=4000) {
			    //printf("got past break ");
		        loggingmode = HALT;
			    //printf("\n\rStop logging sensor data\n\r-");
			    putfs(mainmess2);
				GREEN_LED(OFF);
			    // wait until switch is released
			    while(!(PINB&(1<<PINB0))) ;
			    Delaymsec(200);
			    }
			}
		
		// If logging hasn't started see if button pressed
		if((armpb0==YES)&&(!(PINB&(1<<PINB0)))) {
		    loggingmode = START;
	 		Init_BMP085_Readings();
			//printf("\n\rStart logging sensor data");
			putfs(mainmess1);
			GREEN_LED(ON);
			if(FlashBlankCheck()==YES) {
				RED_LED(ON);
				FlashChipErase();
				RED_LED(OFF);
				}
			elapsedtime = 0;
			if(terminationmode==TIMEOUT) rtccountdown4 = timeoutperiod;
			printf("\n\r-");
			bufferidptr = 1;
			pagecnt = 1;
			flashaddress = 0;
			// Halt until switch is released
			while(!(PINB&(1<<PINB0))) ;
			Delaymsec(200);
			}

		if(timedstartmode==YES) {
		    //printf("\n\rTimed start mode activated\n\r");
			if((CheckStartTime()==YES)) {
				loggingmode = START;
		 		Init_BMP085_Readings();
				//printf("\n\rStart logging sensor data");
				putfs(mainmess1);
				GREEN_LED(ON);
				if(FlashBlankCheck()==YES) {
					RED_LED(ON);
					FlashChipErase();
					RED_LED(OFF);
					}
				elapsedtime = 0;
			    if(terminationmode==TIMEOUT) rtccountdown4 = timeoutperiod;
				printf("\n\r-");
				bufferidptr = 1;
				pagecnt = 1;
				flashaddress = 0;
				}
		    }
		}


	// Handle microSD start conditions - stop conditions are handled by log_2_MicroSD
	//   keyboard start, PB0 start, timed start
    else {
		if((usdpresent==YES)&&(storageselection==MICROSD)) {
			// On power mode
			if(startloggingmode==ONPOWERUP) {
				if(inchar=='q') {
				    startloggingmode = NOTONPOWERUP;
					putCRLFP();
					}
			    else {
					if(rtccountdown2)  printf("Power up mode active  - q to quit - %d seconds to start   \r",rtccountdown2);
					if(!rtccountdown2) {
						bmp085mode = ACTIVE;
						//printoutput = YES;	
						Log_2_MicroSD('t');
						// Disable start logging mode
						//startloggingmode = NOTONPOWERUP;
						putCRLFP();
					    }
					}
			    }
			// Timed start mode
			else if((timedstartmode==YES)&&(logrestart==NO))  {
				if((CheckStartTime()==YES)) {
					bmp085mode = ACTIVE;
					//printoutput = YES;	
					Log_2_MicroSD('t');
					// Disable start logging mode
					//startloggingmode = NOTONPOWERUP;
					putCRLFP();
			    	}
				}
			// If logging hasn't started see if button pressed
			else if((armpb0==YES)&&(!(PINB&(1<<PINB0)))) {
				bmp085mode = ACTIVE;
				//printoutput = YES;	
				Log_2_MicroSD('t');
				// Disable start logging mode
				//startloggingmode = NOTONPOWERUP;
				putCRLFP();
				}
			// If start log command activated do it
			else if(loggingmode==START) {
				printf("Logging started ");
				// if file name mode is increment get the next file name
				if(usdfilenamemode=='i') {
	    		    usdfilenamenumber = ReadEepromInt(usdfilenamenumberoffset);
					sprintf(microsdname,"imu%05d.txt",usdfilenamenumber);
					}
				if(storageselection==MICROSD) printf("to %s",microsdname);
				if(terminationmode==TIMEOUT)  printf(" ending after %d seconds\n\r",timeoutperiod);
				else if(terminationmode==BUTTON) printf(" end by button press\n\r");
				else printf(" end by HALT command\n\r");
				bmp085mode = ACTIVE;
				Log_2_MicroSD('t');
				putCRLFP();
				}
			}
		}

}

// Wait for OK from Zigbee - break if no OK after max time
// Echo characters if verbose mode
// Return 0 if OK received, 1 if ERROR returned or 2 if timeout
int WaitForOKERROR(char mode,int maxdelayperiod)
{
	char inchar;
	unsigned char matchcnt=0;

	mseccountdown2 = maxdelayperiod;

	while((matchcnt<2)&&(mseccountdown2)) {
	    while(!(inchar=getcharnw1())) {
			if(!mseccountdown2) return 1;
			}
		if(mode==VERBOSE) putchar(inchar);
		if((inchar=='O')&&(matchcnt==0)) {
		    matchcnt++;
		    while(!(inchar=getcharnw1())) {
				if(!mseccountdown2) return 1;
				}
			if(mode==VERBOSE) putchar(inchar);
			if(inchar=='K')  return 0;
			}
		}
	return 2;
}

void putfs_lcd(const char *signon1)
{ 	
	char inchar;
    int i;
	
	i = 0;
	while((inchar=pgm_read_byte(&signon1[i]))!=0x00)  messageBuf[i++] = inchar;
	TWI_Start_Write(SMARTLCDTWIADDRESS,messageBuf,i);
}

void SignOn_lcd(void)
{
#ifdef ATMEGA644TARGET
	putfs_lcd(signon1A);
#endif
#ifdef ATMEGA644PTARGET
	putfs_lcd(signon1B);
#endif
#ifdef ATMEGA1284PTARGET
	putfs_lcd(signon1C);
#endif
	
}

void Write_SmartLCD_DisplayLed(unsigned char twiaddress,unsigned char state)
{
	messageBuf[0] = ESC;
	if(state=='o') messageBuf[1] = 'o';
	else messageBuf[1] = 'f';
	TWI_Start_Write(twiaddress,messageBuf,2);
}

void Write_SmartLCD_Int(unsigned char twiaddress,unsigned char row,unsigned char column,int ivalue)
{
 	 char outstrg[20];
	 
	 messageBuf[0] = ESC;
	 messageBuf[1] = 'r';
	 messageBuf[2] = row;
	 messageBuf[3] = ESC;
	 messageBuf[4] = 'p';
	 messageBuf[5] = column;
	 messageBuf[6] = 0;
	 sprintf(outstrg,"%d",ivalue);
	 strcat(messageBuf,outstrg);
	 //printf("\n\rmessagebuf=%s len=%d\n\r",messageBuf,strlen(messageBuf));
	 TWI_Start_Write(twiaddress,messageBuf,strlen(messageBuf));

}

// Write float value to SmartLCD
void Write_SmartLCD_Float(unsigned char twiaddress,unsigned char row,unsigned char column,float fvalue)
{
 	 char outstrg[20];
	 
	 messageBuf[0] = ESC;
	 messageBuf[1] = 'r';
	 messageBuf[2] = row;
	 messageBuf[3] = ESC;
	 messageBuf[4] = 'p';
	 messageBuf[5] = column;
	 messageBuf[6] = 0;
	 sprintf(outstrg,"%f",fvalue);
	 strcat(messageBuf,outstrg);
	 //printf("\n\rmessagebuf=%s len=%d\n\r",messageBuf,strlen(messageBuf));
	 TWI_Start_Write(twiaddress,messageBuf,strlen(messageBuf));
}

//*************************************************************
//  Main work starts here
//*************************************************************
int main(void)
{
	unsigned char inchar,inchar2,inbyte;
	unsigned char toggle,lowbyte,outbyte;
	int loopsetting,loopsettingfast,loopmode;
	int loopmode2,dataoutputperiod;
	int delaycount,rotaryvalue;
	int delay,loop;
	int delaysec,disk_init;
	long loopcount,ltime,mstemperature,mspressure;

	char zigInfo[20];
	char dummyData[10];
	int ctr = 0;
	int IncDevAddr = 0;		//Incoming Device Address
	int DeliveryMode = 0;	//0 for nonARQ  OR  1 for ARQ 
	int DataPortion = 0;	//number of characters transmitted (max=84)

	union {
	    unsigned char tempbuf[4];
		unsigned long outlong;
		} u;

	delay_time starttime;
		
	stdout = &mystdout;
	stdin = &mystdin;
	
RESTART:
	Init_AVR();
	rxcntmax = totalrxcnt = 0;
	
	loopsetting = 70;   // 
	loopsettingfast = 35;
	loopmode = 0;  toggle = 0;
	loopmode2 = 0; 
	mseccountdown1 = loopsetting;
	mseccountdown4 = loopsettingfast;
	loopcount = 0;
	dataoutputperiod = 10;  // seconds
	//LCD_Switches = 0x00;   // Switch toggle values

	delay = 1;
	
	// Insert copyright - version reference in Flash memory
#ifdef  IMU6410
	strcpy(version,"SOC IMU6410  R1.00 (c) 2013");
#else
	strcpy(version,"SOC IMU6420  R1.00 (c) 2013");
#endif
	
	// Show something is happening
	while(0) {  
		BlinkGreen(5);
		BlinkRed(5);
	}
		
	sei(); // Enable interrupts => enable UART interrupts

	BlinkRed(2);
	BlinkGreen(2);
	
	// This code is here for test if something in the code goes wrong
	while(0) {  
		BlinkRed(100);
		BlinkGreen(100);
	    putchar('U');
		}
	 
	signon();

	// This code is here for test if something in printf driver goes wrong
	loop = 0;
	while(0) { 
	    printf("\n\rloop %d\n\r",loop++); 
		BlinkRed(8);
		BlinkGreen(8);
		signon();
		}


	Release_SPI();
	Init_SPI_Master();

	//printf("\n\r test \n\r");
	get_fattime();  // Initialize fat time to current clock
	// See if microSD installed
	if((disk_init=disk_initialize(0))==0) {
	    printf("uSD found");
		if(fatmounted==NO) {
		    f_mount(0, &Fatfs[0]);
			fatmounted = YES;
			printf(" and mounted");
			}
	    usdpresent = YES;

		// See if logging mode starts on power up and delay start for 30 seconds
		// This only applies to microSD
		if((startloggingmode==ONPOWERUP)&&(storageselection==MICROSD)) {
		    // If logrestart mode active start logging immediately
		    if((logrestart==YES)&&(logrestartcnt==0)) {
			     rtccountdown2 = 10;
				 if(delaystarttimesec<rtccountdown2)  rtccountdown2 = delaystarttimesec;
				 }
			else rtccountdown2 = 0;
			if((timedstartmode==YES)&&((rtccountdown2)<delaystarttimesec+1)) {
				rtccountdown2 = 0;
				starttime = CalculateStartTime(delaystarttimesec+1);
				hourstart = starttime.hour;
				minutestart = starttime.minute;
				secondstart = starttime.second;
				}
			}
		}
	else printf("uSD not found");
	//printf("\n\rdisk_init=%d\n\r",disk_init);

#ifdef ZIGBEEWIRELESS
	Delaymsec(500);  // Let the ZigBee device come alive before talking to it
	checkZigbee();
#endif

	if(i2cmode==ACTIVE) Enumerate_TWI_Devices();
	if(dof10wmode==ACTIVE) CheckforMEMSensors();

	printf("\n\r-");
	Delaymsec(100);
	while(getcharnw1() != 0); //I need to empty the UART1 buffer on startup
	inchar = 0;

	// Wait for commands on serial channel
	while (1) {
		//Delaymsec(1000);
		//printf("\n\rloop %d\n\r",loop++);
		CheckLogging(inchar);


#ifdef	ZIGBEEWIRELESS
	// Process commands coming in over the ZigBee link
	if((inchar = getcharnw1()) == 'D') {
		ctr = 0;
		zigInfo[ctr] = inchar;
		ctr++;
		while((inchar = getchar1()) != ':') {
			zigInfo[ctr] = inchar;
			ctr++;
			}
		zigInfo[ctr] = 0;
			inchar = getchar1();
			putchar(inchar);
			switch (inchar) {
				case '&':
					putchar(inchar);
					goto RESTART;
					break;
				case 'j':
					inchar = getchar1();
					if(inchar == 'j')	signon();
					else	puts("Wrong Command!");
					break;
				case 'd':
					inchar = getchar1();
					if(inchar == 'z')	UploadZigData();
					else	puts("Wrong Command!");
					break;
				case 'c':
					inchar = getchar1();
					if(inchar == 'k')	checkZigbee();
					else	puts("Wrong Command!");
					break;
				case 'z':
					inchar = getchar1();
					if(inchar == 's')	ZigbeeSurvey();
					else	puts("Wrong Command!");
					break;
				case '3':
					inchar = getchar1();
					if(inchar == 'i')	initZigbeeSlave();
					else	puts("Wrong Command!");
					goto RESTART;
					break;
				case 's':
					inchar = getchar1();
					if(inchar == 't')	set2Time();
					else if(inchar == 'd')	set2Date();
					else	puts("Wrong Command!");
					break;
				case 'g':
					inchar = getchar1();
					if(inchar == 't')	get2Time();
					else if(inchar == 'd')	get2Date();
					else	puts("Wrong Command!");
					break;
				// Output 9 or 10 DOF data continuoulsy using format in inchar2
				case '9':
					//printoutput = YES;	
					bmp085mode = INACTIVE;
					inchar = getchar1();
					if(inchar == 's')	{
						DisplayDOF10WData(inchar);
						puts1("atd 0\r");
						puts1(Buff);
						putchar1('\r');
						WaitForOKERROR(VERBOSE,1000);
						}
					else {
						Log_2_MicroSD(inchar);
						}
					break;
				case 'l':
					inchar = getchar1();
					if(inchar == 'm')	{
						Uart1Channel = YES;
						LogDataToSerialFlash();
						Uart1Channel = NO;
						}
					break;
				case 'k':
					inchar = getchar1();
					//  Check if uSD card is installed - if not setup to log 
					//  to serial flash - note this only happens on IMU6410's
					if(inchar == 'p')	{
					 	StorageDeviceMenuTest();
					 	SaveEepromPreferences();
						}
					if(storageselection==MICROSD) {
						GREEN_LED(ON);

						//  This code is for serial flash only
						if(FlashBlankCheck()==YES) {
						    RED_LED(ON);
						    FlashChipErase();
							RED_LED(OFF);
							}
						bufferidptr = 1;
						pagecnt = 1;
						flashaddress = 0;

					    loggingmode = START;
						if(terminationmode==TIMEOUT) rtccountdown4 = timeoutperiod;
						printf("\n\rLogging mode started");
						Init_BMP085_Readings();
						}
					break;
				default:
					puts("Wrong Command: ");
					putchar(inchar);
					puts(" does not exist!");
					break;
				}
			//sscanf(zigInfo, "%s %d,%d,%d", dummyData, &IncDevAddr, &DeliveryMode, &DataPortion);
			//printf("\n\rIncoming Device Address: %d\n\rDelivery Mode: %d\n\rNumber of characters: %d",
			//IncDevAddr, DeliveryMode, DataPortion);
			putCRLFP();
		}
		
#endif

		// Work starts here
		if((inchar = getcharnw())) {

				// If escmode is on ignore all commands except ESC
				if((escmode==ON)&&(inchar!=ESC)) inchar = 0;
				 
		        if(0) ProcessLCDCommands(inchar);
				
			   	switch (inchar) {
					 case (ESC):
						 inchar = getchar();
						 if(inchar=='d') {
						     dadmode = ACTIVE;
							 DAD_Controller();
							 printf("\n\rExit DAD Mode\n\r>");
							 loopsetting = 70;   // 
							 loopsettingfast = 35;
							 }
						 else if(inchar==ESC) {
					        if(escmode==ON) {
							    escmode = OFF;
								printf("\n\rESC Mode inactive\n\r");
								SaveEepromPreferences();
								}
							else {
							    escmode = ON;
								printf("\n\rESC Mode active - all further commands ignored\n\r");
								printf("Enter ESC to deactivate ESC mode\n\r");
								SaveEepromPreferences();
								}
							}
						 else printf("\n\rUnknown ESC command %c",inchar);
						 break;
					case 'e': 
					     putchar(inchar);
						 Enumerate_TWI_Devices();
					     putchar('-');
						 break;
					case '&':
					     putchar(inchar);
						 goto RESTART;
						 break;
					case 'v': case 'V':
					     putchar(inchar);
						 BatteryVoltage();
					     putchar('-');
						 break;
					case 'S':
						 printf("\n\rPrint or reset EEPROM preferences [p,r]: ");
						 putchar(inchar = getchar());
						 if(inchar=='p') PrintEepromPreferences();
						 else {
						 	printf("\n\rReset EEPROM preference to defaults\n\r");
							WriteEepromSignature();
						 	LoadEepromPreferences();
							SetDefaultEepromPreferences();
							SaveEepromPreferences();
							}
						 putchar('-');
						 break;
					case 's':
					     putchar(inchar);
						 if(1) {
							 signon();
							 if(i2cmode==ACTIVE) {
							     Enumerate_TWI_Devices();
								 //Enumerate_Other_TWI_Devices();
								 }
							 if(dof10wmode==ACTIVE) CheckforMEMSensors();
							 //if(smartlcdstate)  SignOn_lcd();
							 putCRLFP();
						 }
						 else {
						     printf("Enter TWI devices to enumerate\n\r");
							 while((inchar=getcharnw())!='e') {
							     if(inchar!=0) putchar(inchar);
							     if(inchar=='b') Enumerate_BMP085();
								 else if(inchar=='g') Enumerate_ITG3200();
								 else if(inchar=='h') Enumerate_HMC5883();
								 else if(inchar=='a') Enumerate_BMA180();
								 else if(inchar=='s'){
									 signon();
									 if(i2cmode==ACTIVE) {
									     Enumerate_TWI_Devices();
										 //Enumerate_Other_TWI_Devices();
										 }
									 if(dof10wmode==ACTIVE) CheckforMEMSensors();
									 putCRLFP();
									 }
								 }
							 printf("\n\r-");
							 }
						 break;
					case 'm': case 'M':
						  putcharr(inchar);
						  MeasureAnalogInputs();
					      putchar('-');
						  break;
					case 'd':
						  putcharr(inchar);
						  DisplayMemory();
					      putchar('-');
						  break;
					case 'c':
						  putchar(inchar);
						  inchar2 = getchar();
						  printf("%c\n\r",inchar2);
						  switch (inchar2) {
						      case 's':
								  putfs(mainmess3);
								  delay = readint();
								  break;
						      case 'b':
								  putfs(mainmess4);
								  bmp085mode = getchar();
								  break;
							  default:
							      break;
						      }
					      putCRLFP();
						  break;
					case 'p':
							putcharr(inchar);
						    LoadMemory();
							putchar('-');
							break;
					case 'a':
						    DumpAnalogChs(EXTERNALREF);
							putcharr('-');
							break;
					case 'A':
						    printf("\n\r2.56v ref = %d\n\r",AnalogCh(7,INTERNALREF256V));
						    printf("1.1v ref = %d\n\r",AnalogCh(7,INTERNALREF11V));
							putchar('-');
							break;
					case '4':
					        putchar('4');
							inchar2 = getchar();
					        printf("%c\n\r",inchar2);		
							//  q - process noise
							//  r - measurement noise
							//  p - covariance 
							//kalman_init(double q, double r, double p, Read_BMP085_pressurefloat(QUIET));					
							filtered_pressure = kalman_init(0.0008,0.003,1.0,Read_BMP085_pressurefloat(QUIET));					
							pressuremin = 20.0;
							pressuremax = 10.0;
							while((inchar=getcharnw())!='e') DisplayBMP085Brief(inchar2);
							printf("Pressure min = %.5f max = %.5f\n\r",pressuremin,pressuremax);
							putCRLFP();
							break;
					case 'i':
					        putchar('i');
					        IMU6410Menu();
							putCRLFP();
							break;
					case 'I':
					        putchar('i');
					        DOF10WMenu();
							putCRLFP();
							break;
					case '@':
					        putchar('@');
					        Execute_TWI_Command();
							putCRLFP();
							break;
					case 'w':
					        //printf("w\n\rClear CRA7 bit");
							putfs(mainmess10);
							lowbyte = Read_HMC5883_Register(HMC5883_CONFIG_REGA);
							Write_HMC5883_Register(HMC5883_CONFIG_REGA,(lowbyte&0x7F));
							putCRLFP();
							break;
					case '`':
						    //printf("`\n\rSet CRA7 bit");
							putfs(mainmess11);
							Write_HMC5883_Register(HMC5883_CONFIG_REGA,0xf0);
							putCRLFP();
							break;
					// Output 10 DOF MEMs data continuously using format in inchar2
					case '1':
					        putchar('1');
							putchar((inchar=getchar()));
							if(inchar!='0') break;
							inchar2 = getchar();
							printf("%c\n\r",inchar2);		
							bmp085mode = ACTIVE;
							printoutput = YES;	
	
							if(inchar2=='s')  DisplayDOF10WData(inchar2);
							else Log_2_MicroSD(inchar2);

							putCRLFP();
							break;
  					// Output 9 or 10 DOF data continuoulsy using format in inchar2
					case '9':
					        putchar('9');
							inchar2 = getchar();
							printf("%c\n\r",inchar2);
							printoutput = YES;	
							bmp085mode = INACTIVE;
							
							if(inchar2=='s') DisplayDOF10WData(inchar2);	
							else Log_2_MicroSD(inchar2);

							putCRLFP();
							break;
					case '2':
							DDRA = 0x2C;
							ClrBit(PORTA,PIN5);  // Hold ANT in reset
							SetBit(PORTA,PIN2);  // Deactive ANT SUSPEND
							ClrBit(PORTA,PIN3);  // Deactivate ANT SLEEP
							putchar1(0x00);  // put serial line is know state
							Delaymsec(2);   // wait for character to transmit
							SetBit(PORTA,PIN5);  // Release ANT reset

					        printf("2\n\rWrite message to ANT - type e to exit\n\r");
					        //printf("Hit ESC to exit\n\r");
							while((inchar=getchar())!='e') {
							    //putchar(inchar);
							    putchar1(0xa4);
							    putchar1(0x01);
							    putchar1(0x4b);
							    putchar1(0x01);
							    putchar1(0xef);
								}
							inchar = 0;
							putCRLFP();
							break;
					case 'z':
							printf("3\n\rZigBee configuration/test commands\n\r");
							printf("Enter i-init,p-pass thru,d-data dump,c to send characters,e-exit:\n\r");
							// AT+WLEAVE
							// ATX
							// AT+GSN=3
							// AT+WPANID=1620
							// AT+WCHMASK=100000
							// AT+WROLE=2 +WSRC=56
							// AT+WAUTONET=1 Z
							while((inchar=getchar())!='e') {
								putchar(inchar);
								if(inchar=='p') {
							        printf("p\n\rPass through mode - ESC to exit\n\r");
							        //printf("Hit ESC to exit\n\r");
									while((inchar=getcharnw())!=ESC) {
									    if(inchar!=0) putchar1(inchar);
										if(inchar=getcharnw1()) putchar(inchar);
										//Delaymsec(10);
										}
									}
								if(inchar=='d') {
							        printf("p\n\rData mode atd 0\rIMU6410\r");
							        puts1("ATD 0\rIMU6410\r");
									Delaymsec(200);
									while((inchar=getcharnw1())) putchar(inchar);
									}
								if(inchar=='c') {
							        printf("c\n\rWrite characters to UART1 type e to exit\n\r");
							        //printf("Hit ESC to exit\n\r");
									while((inchar=getchar())!='q') {
									    putchar1(inchar);
										Delaymsec(10);
										while((inchar=getcharnw1())) {
										    putchar(inchar);
											}
										}
									}
								else if(inchar=='i') {
									printf("\n\rEnter node id: ");
									scanf("%s",&dummyData);
									printf("%s\n\r",dummyData);
									puts1("AT+WLEAVE\r");
									WaitForOKERROR(VERBOSE,1000);
									while((inchar=getcharnw1())) putchar(inchar);
								    puts1("ATX\r");
									WaitForOKERROR(VERBOSE,1000);
									while((inchar=getcharnw1())) putchar(inchar);
									puts1("AT+GSN=3\r");
									WaitForOKERROR(VERBOSE,1000);
									while((inchar=getcharnw1())) putchar(inchar);
									puts1("AT+WPANID=1620\r");
									WaitForOKERROR(VERBOSE,1000);
									while((inchar=getcharnw1())) putchar(inchar);
									puts1("AT+WCHMASK=100000\r");
									WaitForOKERROR(VERBOSE,1000);
									while((inchar=getcharnw1())) putchar(inchar);
									puts1("AT+WROLE=2 +WSRC=");
									puts1(dummyData);
									puts1("\r");
									WaitForOKERROR(VERBOSE,1000);
									while((inchar=getcharnw1())) putchar(inchar);
									puts1("AT+WAUTONET=1 Z\r\r");
									WaitForOKERROR(VERBOSE,4000);
									while((inchar=getcharnw1())) {
										Delaymsec(10);
									    putchar(inchar);
										}
									}
								printf("\n\rEnter i-init,p-pass thru,d-data dump,c to send characters,e-exit:\n\r");
								}
							putCRLFP();
							break;
					case '5':
					        printf("4\n\rEnter delay start time in sec or enter 0 sec to exit\n\r");
					        //printf("Hit ESC to exit\n\r");
							delaysec = 1;
							while(delaysec) {
							    localecho = YES;
								scanf("%d",&delaysec);
								localecho = NO;
								getchar();
								starttime = CalculateStartTime(delaysec);
								printf("\n\rCurrent time: %d %d %d ",hours,minutes,seconds);
								printf("Delay time: %d %d %d\n\r",starttime.hour,starttime.minute,starttime.second);
								}
							putCRLFP();
							break;
					case '6':
							printf("6");
							//Write_SmartLCD_DisplayLed(SMARTLCDTWIADDRESS,'o');
							float a = 12.34;
							float b = 54.456;
							int c = 120;
							int d = 3456;
							while(!(inchar=getcharnw())) {
							    Write_SmartLCD_Float(SMARTLCDTWIADDRESS,1,1,a);
								Delaymsec(2);
							    Write_SmartLCD_Float(SMARTLCDTWIADDRESS,2,2,b);
								Delaymsec(2);
							    Write_SmartLCD_Int(SMARTLCDTWIADDRESS,3,3,c);
								Delaymsec(2);
							    Write_SmartLCD_Int(SMARTLCDTWIADDRESS,4,4,d);
								a += 0.123;
								b += 0.045;
								c += 1;
								d -= 1;
							    Delaymsec(1000);
							    }
							break;
					case '7':
							printf("7");
							messageBuf[0] = ESC;    // Escape command
							messageBuf[1] = 'g';	// Load custom character array
							messageBuf[2] = 0;		// Load into char 0
							messageBuf[3] = 0x1F;
							messageBuf[4] = 0x1F;
							messageBuf[5] = 0x1F;
							messageBuf[6] = 0x1F;
							messageBuf[7] = 0x1F;
							messageBuf[8] = 0x1F;
							messageBuf[9] = 0x1F;
							TWI_Start_Write(SMARTLCDTWIADDRESS,messageBuf,10);
							for(outbyte=0x00; outbyte<0x04; outbyte++) {
							    messageBuf[0] = outbyte;
							    TWI_Start_Write(SMARTLCDTWIADDRESS,messageBuf,1);
								Delaymsec(100);
								}
						    break;
					case '8':
							printf("8\n\r");
							while(!(inchar=getcharnw())) {
								inbyte = Read_SmartLCD_Switches(SMARTLCDTWIADDRESS);
								Delaymsec(4);
								rotaryvalue = Read_SmartLCD_Rotaryvalue(SMARTLCDTWIADDRESS);
								printf("switch = %x rotary = %d  \r",inbyte,rotaryvalue);
								//printf("%d  ",rotaryvalue);
								Delaymsec(4);
								}
							printf("\n-");
						    break;
					case 'b':
					        printf("b\n\rStart stk500v2.02 bootloader!\n\r");
						    stk500v2_bootloader();
							break;
					case 'B':
					        printf("b\n\rStart stk500v2 bootloader!\n\r");
						    stk500v2_bootloaderB();
							break;
					case 'g': 
						    getTime();
							printf("\n-");
							break;
				    case 't': 
						    getsettime(VERBOSE);
							printf("\n-");
							break;
				    case 'T': 
						    getsetdate(VERBOSE);
							printf("\n-");
							break;
					case 'E':
							putchar('E');
							FlashChipErase();
							putCRLFP();
							break;
					case 'l':  case 'L':
							putchar('l');
							LogDataToSerialFlash();
							putCRLFP();
							break;
					case 'F':
						    FAT32_FileSystem_Test();
							printf("\r-");
							break;
					case 'f':
							putchar(inchar);
					        if(fatmounted==YES) {
							    Dump_Directory_Listing();
								printf("\r-");
								}
							else printf("\n\rFAT not mounted - use F command\n\r-");
							
							break;
#ifdef MS5611PRESENT
					case '=':
							//printf("\n\rraw p=%ld",Read_MS5611_UnSigned(MS5611_CONVERTD14));
							//printf("\n\rraw t=%ld",Read_MS5611_UnSigned(MS5611_CONVERTD24));
							messageBuf[0] = MS5611_RESET;
							TWI_Start_Write(MS5611TWIADDRESS,messageBuf,1);
							Delaymsec(10);

							messageBuf[0] = MS5611_CONVERTD14;
						    TWI_Start_Write(MS5611TWIADDRESS,messageBuf,1);
							Delaymsec(13);
							// read ADC
							messageBuf[0] = MS5611_ADC;
						    TWI_Start_Write(MS5611TWIADDRESS,messageBuf,1);
							//inchar = getchar();
							messageBuf[0] = 0;
							messageBuf[1] = 0;
							messageBuf[2] = 0;
							messageBuf[3] = 0;
							TWI_Start_Read(MS5611TWIADDRESS,3);
						    TWI_Get_Data_From_Transceiver(messageBuf,3);
							u.outlong = 0;
							u.tempbuf[3] = 0;
							u.tempbuf[2] = messageBuf[0];
							u.tempbuf[1] = messageBuf[1];
							u.tempbuf[0] = messageBuf[2];
							mspressure = u.outlong;

							messageBuf[0] = MS5611_CONVERTD24;
						    TWI_Start_Write(MS5611TWIADDRESS,messageBuf,1);
							Delaymsec(13);
							// read ADC
							messageBuf[0] = MS5611_ADC;
						    TWI_Start_Write(MS5611TWIADDRESS,messageBuf,1);
							//inchar = getchar();
							messageBuf[0] = 0;
							messageBuf[1] = 0;
							messageBuf[2] = 0;
							messageBuf[3] = 0;
							TWI_Start_Read(MS5611TWIADDRESS,3);
						    TWI_Get_Data_From_Transceiver(messageBuf,3);
							u.outlong = 0;
							u.tempbuf[3] = 0;
							u.tempbuf[2] = messageBuf[0];
							u.tempbuf[1] = messageBuf[1];
							u.tempbuf[0] = messageBuf[2];
							mstemperature = u.outlong;

							long dt,temp,p;
							long long off,sens;
							float psi;

							dt = mstemperature - (((long)ms5611_c5)*256);
							temp = 2000 + (dt*(unsigned long)ms5611_c6)/8388608;
							printf("\n\rdt = %ld  temp=%#6.2f  ",dt,(float)temp/100.0);

							off = (unsigned long long)ms5611_c2*65536 + ((unsigned long long)ms5611_c4*dt)/128;
							sens = (unsigned long long)ms5611_c1*32768 + ((unsigned long long)ms5611_c3*dt)/256;
							p = (mspressure * (sens/2097152) - off)/32768;
							psi = ((float)p/100000.0)*14.503773773;
							printf(" p = %ld  %#6.4f  %6.1f",p,psi,(14.585-psi)/0.000407);

							putCRLFP();
							break;
#endif
					case CR:
							if(flashprogrammingactive==NO)  putCRLFP();
							break;
					default:
							putchar(inchar);
							//Delaymsec(20);
							break;
					}
				}

		
		if((rtccountdown4==0)&&(rtccnt==YES)) {
		    printf(" timeout\n\r");
			rtccnt = NO;
			}
			
		// Blink green led slowly to show AVR is alive
		if(mseccountdown1==0) {
			mseccountdown1 = loopsetting;
			//if(loggingmode==START) mseccountdown1 = loopsetting/4;
			//else mseccountdown1 = loopsetting;
			if(loopmode)  {
			    if(loggingmode==START) GREEN_LED(ON);
				else GREEN_LED(ON);
				loopmode = 0;
				}
			else {
				if(loggingmode==START) GREEN_LED(OFF);
				else GREEN_LED(OFF);
				loopmode = 1;
				}
			}
		// Blink red led fast to show AVR is alive
		if(mseccountdown4==0) {
			mseccountdown4 = loopsettingfast;
			if(loopmode2)  {
			    if(loggingmode==START) RED_LED(ON);
				else RED_LED(ON);
				loopmode2 = 0;
				}
			else {
				if(loggingmode==START) RED_LED(OFF);
				else RED_LED(OFF);
				loopmode2 = 1;
				}
			}
										
		// Clean up flash state machine if no new Intel
		// Hex records arrive for 3 seconds
		if((flashcountdown==0)&&(flashprogrammingactive==YES))
				CleanUpFlashState();
		}
}
